@@ -26,6 +26,48 @@ $(function() {
 	var $helpDialog = $("#help-dialog");
 	var $perlDocDialog = $("#perl-doc-dialog");
 
+	window.whitespaceOverlay = {
+		showTabs   : false,
+		showSpaces : false,
+		token      : function(stream) {
+
+			if (stream.eatWhile(/\S/)) {
+				//Eat all non-whitespace tokens with no whitespace highlighting
+				return null;
+			}
+
+			// Consume the whitespace character
+			ch = stream.next();
+
+			// And give it the proper CSS class
+			if(ch == '\t') {
+				return this.showTabs ? "whitespace-tab" : null;
+			} else {
+				return this.showSpaces ? "whitespace-space" : null;
+			}
+		}
+	};
+
+	window.showWhitespace = function(editor) {
+		whitespaceOverlay.showSpaces = $("#show_spaces_checkbox").is(':checked');
+		whitespaceOverlay.showTabs = $("#show_tabs_checkbox").is(':checked');
+		
+		// Refresh...
+		editor.removeOverlay(window.whitespaceOverlay);
+		editor.addOverlay(window.whitespaceOverlay);
+	};
+
+	// Create the readonly CodeMirror input editor instance
+	var inputEditor = CodeMirror.fromTextArea(document.getElementById('input'), {
+		//TODO these should be configurable!
+		tabSize: 4,
+		indentUnit: 4,
+		indentWithTabs: true
+	});
+
+	// Some friendly advice
+	inputEditor.setValue("The following input will be sent to the process that is run");
+
 	// Create the readonly CodeMirror output editor instance
 	var outputEditor = CodeMirror.fromTextArea(document.getElementById('output'), {
 		//TODO these should be configurable!
@@ -226,6 +268,7 @@ $(function() {
 			styleSelectedText: true,
 			autoCloseBrackets: true,
 			styleActiveLine: true,
+			flattenSpans: false,
 			extraKeys: {
 				"F1": function(cm) {
 					displayHelp(cm);
@@ -323,6 +366,10 @@ $(function() {
 			} else {
 				$("#pod").html('Not supported for mode "' + mode + '"');
 			}
+			
+			window.showWhitespace(editor);
+
+			editor.refresh();
 
 		}, 0);
 
@@ -415,7 +462,19 @@ $(function() {
 	window.GetOutputEditor = function() {
 		return outputEditor;
 	};
-	
+
+	window.GetInputEditor = function() {
+		return inputEditor;
+	};
+
+	window.GetEditors = function() {
+		var results = [];
+		for(var editorId in editors) {
+			results.push(editors[editorId].editor);
+		}
+		return results;
+	};
+
 	$(".results").hide();
 
 	$(document).on('action-perl-doc', function() {
@@ -630,7 +689,7 @@ $(function() {
 
 			// Load JSHint and then run
 			$.ajax({
-				url: 'assets/jshint/jshint.js',
+				url: 'assets/jshint/jshint-1.0.0.js',
 				dataType: "script",
 				success: function() {
 					jsHint(editor);
